]> git.sesse.net Git - cubemap/blobdiff - stream.h
Add suppor for raw (non-Metacube) inputs over HTTP. Only really useful for TS.
[cubemap] / stream.h
index 4947c3fbd7fdf11a43b8456a78f4a0c80812c58f..057a48cbe08e516be02f756ce4aeac3701672ee2 100644 (file)
--- a/stream.h
+++ b/stream.h
@@ -8,6 +8,7 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <sys/uio.h>
+#include <deque>
 #include <string>
 #include <vector>
 
@@ -23,7 +24,7 @@ struct Stream {
        // Must be in sync with StreamConfig::Encoding.
        enum Encoding { STREAM_ENCODING_RAW = 0, STREAM_ENCODING_METACUBE };
 
-       Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding);
+       Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding, Encoding src_encoding);
        ~Stream();
 
        // Serialization/deserialization.
@@ -50,6 +51,9 @@ struct Stream {
        // be Metacube, for reflecting to another Cubemap instance).
        Encoding encoding;
 
+       // What encoding we expect the incoming data to be in (usually Metacube).
+       Encoding src_encoding;
+
        // The stream data itself, stored in a circular buffer.
        //
        // We store our data in a file, so that we can send the data to the
@@ -75,9 +79,9 @@ struct Stream {
        // than <backlog_size>, since the buffer wraps.
        size_t bytes_received;
 
-       // The last point in the stream that is suitable to start new clients at
-       // (after having sent the header). -1 if no such point exists yet.
-       ssize_t last_suitable_starting_point;
+       // A list of points in the stream that is suitable to start new clients at
+       // (after having sent the header). Empty if no such point exists yet.
+       std::deque<size_t> suitable_starting_points;
        
        // Clients that are in SENDING_DATA, but that we don't listen on,
        // because we currently don't have any data for them.
@@ -93,11 +97,11 @@ struct Stream {
 
        // Queued data, if any. Protected by <queued_data_mutex>.
        // The data pointers in the iovec are owned by us.
-       std::vector<iovec> queued_data;
-
-       // Index of the last element in queued_data that is suitable to start streaming at.
-       // -1 if none. Protected by <queued_data_mutex>.
-       int queued_data_last_starting_point;
+       struct DataElement {
+               iovec data;
+               StreamStartSuitability suitable_for_stream_start;
+       };
+       std::vector<DataElement> queued_data;
 
        // Put client to sleep, since there is no more data for it; we will on
        // longer listen on POLLOUT until we get more data. Also, it will be put
@@ -117,8 +121,14 @@ private:
 
        // Adds data directly to the stream file descriptor, without adding headers or
        // going through <queued_data>.
+       // You should hold the owning Server's <mutex>, and probably call
+       // remove_obsolete_starting_points() afterwards.
+       void add_data_raw(const std::vector<DataElement> &data);
+
+       // Remove points from <suitable_starting_points> that are no longer
+       // in the backlog.
        // You should hold the owning Server's <mutex>.
-       void add_data_raw(const std::vector<iovec> &data);
+       void remove_obsolete_starting_points();
 };
 
 #endif  // !defined(_STREAM_H)